#!/usr/bin/env node

'use strict';

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _gaze = require('gaze');

var _gaze2 = _interopRequireDefault(_gaze);

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _meow = require('meow');

var _meow2 = _interopRequireDefault(_meow);

var _converter = require('./converter');

var _converter2 = _interopRequireDefault(_converter);

var _writer = require('./writer');

var _writer2 = _interopRequireDefault(_writer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cli = (0, _meow2.default)({
  description: 'Creates .flow type definition files from CSS Modules files',
  help: `
Usage
  $ css-modules-flow-types <path> [<path>] [options]

path    directory to search for CSS Modules (or concrete files to convert)

Options
  --watch, -w       Run in watch mode
  --extension, -e   File extension (defaults to "css")
    `
}, {
  boolean: ['watch'],
  string: ['_'],
  alias: {
    e: 'extension',
    h: 'help',
    w: 'watch'
  }
});

const main = () => {
  const watch = cli.flags.watch;


  if (!cli.input || cli.input.length === 0) {
    return cli.showHelp();
  }

  const extension = cli.flags.extension || 'css';

  const filesList = cli.input.length > 1 ? cli.input : null;
  const filePath = cli.input.length === 1 ? cli.input[0] : null;
  const filesPattern = filePath && _path2.default.join(filePath, `**/*.${extension}`);

  const rootDir = process.cwd();
  const converter = new _converter2.default(rootDir);

  function handleFile(filePath) {
    const f = _path2.default.resolve(filePath);
    converter.convert(f).then(content => {
      const outputFilePath = _path2.default.join(f + '.flow');
      return (0, _writer2.default)(outputFilePath, content);
    }).then(outputFilePath => {
      console.log('Wrote ' + _chalk2.default.green(outputFilePath));
    }).catch(reason => console.error(_chalk2.default.red('[Error] ' + reason)));
  }

  if (!watch) {
    const files = filesList ? filesList : _globby2.default.sync(filesPattern);
    files.forEach(handleFile);
  } else {
    if (!filePath) {
      console.error(_chalk2.default.red(`Watch mode requires a single path... Not ${filesList}`));
      return;
    }
    (0, _gaze2.default)(filesPattern, function (err, files) {
      this.on('changed', handleFile);
      this.on('added', handleFile);
    });
  }
};

main();