'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _cssModulesLoaderCore = require('css-modules-loader-core');

var _cssModulesLoaderCore2 = _interopRequireDefault(_cssModulesLoaderCore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Sorts dependencies in the following way:
// AAA comes before AA and A
// AB comes after AA and before A
// All Bs come after all As
// This ensures that the files are always returned in the following order:
// - In the order they were required, except
// - After all their dependencies
const traceKeySorter = (a, b) => {
  if (a.length < b.length) {
    return a < b.substring(0, a.length) ? -1 : 1;
  } else if (a.length > b.length) {
    return a.substring(0, b.length) <= b ? -1 : 1;
  } else {
    return a < b ? -1 : 1;
  }
}; /*
   Notice:
   
   This file was originally forked from: https://raw.githubusercontent.com/css-modules/css-modules-loader-core/master/src/file-system-loader.js
   
   See:
   https://github.com/Quramy/typed-css-modules/blob/master/src/fileSystemLoader.js
   */
/* this file is forked from https://raw.githubusercontent.com/css-modules/css-modules-loader-core/master/src/file-system-loader.js */
/* eslint-disable prefer-const, no-empty, prettier/prettier */
// prettier-ignore

class FileSystemLoader {
  constructor(root, plugins) {
    this.root = root;
    this.sources = {};
    this.importNr = 0;
    this.core = new _cssModulesLoaderCore2.default(plugins);
    this.tokensByFile = {};
  }

  fetch(_newPath, relativeTo, _trace, initialContents) {
    let newPath = _newPath.replace(/^["']|["']$/g, ''),
        trace = _trace || String.fromCharCode(this.importNr++);
    return new Promise((resolve, reject) => {
      let relativeDir = _path2.default.dirname(relativeTo),
          rootRelativePath = _path2.default.resolve(relativeDir, newPath),
          fileRelativePath = _path2.default.resolve(_path2.default.join(this.root, relativeDir), newPath);

      // if the path is not relative or absolute, try to resolve it in node_modules
      if (newPath[0] !== '.' && newPath[0] !== '/') {
        try {
          fileRelativePath = require.resolve(newPath);
        } catch (e) {}
      }

      if (!initialContents) {
        const tokens = this.tokensByFile[fileRelativePath];
        if (tokens) {
          return resolve(tokens);
        }

        _fs2.default.readFile(fileRelativePath, 'utf-8', (err, source) => {
          if (err && relativeTo && relativeTo !== '/') {
            resolve([]);
          } else if (err && (!relativeTo || relativeTo === '/')) {
            reject(err);
          } else {
            this.core.load(source, rootRelativePath, trace, this.fetch.bind(this)).then((_ref) => {
              let injectableSource = _ref.injectableSource,
                  exportTokens = _ref.exportTokens;

              this.sources[trace] = injectableSource;
              this.tokensByFile[fileRelativePath] = exportTokens;
              resolve(exportTokens);
            }, reject);
          }
        });
      } else {
        this.core.load(initialContents, rootRelativePath, trace, this.fetch.bind(this)).then((_ref2) => {
          let injectableSource = _ref2.injectableSource,
              exportTokens = _ref2.exportTokens;

          this.sources[trace] = injectableSource;
          this.tokensByFile[fileRelativePath] = exportTokens;
          resolve(exportTokens);
        }, reject);
      }
    });
  }

  get finalSource() {
    return Object.keys(this.sources).sort(traceKeySorter).map(s => this.sources[s]).join('');
  }

  clear() {
    this.tokensByFile = {};
    return this;
  }
}
exports.default = FileSystemLoader;